@@ -55,7 +55,7 @@ _delay_import_(globals(),"dimensions")
 _delay_import_(globals(),"dimpaths")
 _delay_import_(globals(),"casts")
 _delay_import_(globals(),"type_attribute_freeze")
-
+_delay_import_(globals(),"..utils.missing","Missing")
 
 #}}}
 
@@ -109,7 +109,20 @@ class Type(object):#{{{
     def toDefval(self):
         """Returns default value."""
         return self._defval
-   
+  
+    def toMissingval(self):
+        return Missing
+    
+    def hasMissingValInfo(self):
+        try:
+           self.toMissingval()
+           return True
+        except RuntimeError:
+           return False
+
+    def hasMissing(self):
+        return self.has_missing
+
     def getName(self):
         """Returns base type name"""
         return self.name
@@ -297,7 +310,15 @@ class TypeAny(TypeUnknown):#{{{
     def copy(self, **newattr):
         """Returns copy of this type"""
         return copy.copy(self)
-    
+
+    def setHasMissing(self, value):
+        if self.has_missing != value:
+            res = self.copy()
+            res.has_missing = value
+        else:
+            res = self
+        return res
+
     def __repr__(self):
         res = self.name
         if(self.has_missing):
@@ -497,6 +518,49 @@ class TypeRecordDict(TypeTuple):#{{{
 
 addType(TypeRecordDict)#}}}
 
+class TypeIndexDict(TypeTuple):#{{{
+    name = "index_dict"
+    
+    def __init__(self, has_missing=False, subtypes=(), 
+                       fieldnames=("key","value")):
+        """
+        Creates type object.
+
+        :param has_missing: bool, optional
+        :param subtypes: tuple(type objects, i.e. subclasses of TypeUnknown), optional
+        :param fieldnames: tuple(strings), optional
+        """
+
+        assert isinstance(subtypes, tuple), \
+                "The subtypes argument should be a tuple"
+
+        assert all(isinstance(fieldname, basestring) for fieldname in fieldnames), \
+                "Fieldnames should be strings"
+
+        assert not fieldnames or len(fieldnames) == len(subtypes), \
+            "Length of fieldnames should be equal to length of tuples (or empty"
+
+        assert len(subtypes) == 2, "IndexDict should have a key and value subtype"
+
+        TypeAny.__init__(self, has_missing)
+        self.subtypes = subtypes
+        self.fieldnames = fieldnames
+
+    def __repr__(self):
+        res = 'dict(' 
+        if(len(self.fieldnames) == len(self.subtypes)):
+            res += ": ".join((fname + "=" + str(subtype) 
+                for fname, subtype in zip(self.fieldnames, self.subtypes)))
+        else:
+            res += ": ".join((str(subtype) 
+                    for subtype in self.subtypes))
+        res += ')'
+        if(self.has_missing):
+            res += "?"
+        return res
+
+addType(TypeIndexDict)#}}}
+
 #pylint: disable-msg=E1101
 class TypeArray(TypeAny):#{{{
     """Type representing a collection of values, 
@@ -523,9 +587,12 @@ class TypeArray(TypeAny):#{{{
                 "Dims tuple should contain Dim objects"
         assert isinstance(subtypes, tuple) and len(subtypes) == 1, \
                 "One subtype should be set for array type"
-        if dims:
-            dims[0].has_missing = has_missing
-
+        
+        has_missing = dims[0].has_missing or has_missing
+        if self.__class__ == TypeArray:
+            assert not has_missing, "Array type not allowed to have missing values"
+        
+        dims[0].has_missing = has_missing
         self.subtypes = subtypes
         self.dims = dims
         TypeAny.__init__(self, has_missing)
@@ -548,28 +615,47 @@ class TypeArray(TypeAny):#{{{
             res = cls(has_missing=type1.has_missing or type2.has_missing, dims=dims, subtypes=tuple(subtypes)) 
         return res
 
+    def setHasMissing(self, value):
+        s = self.subtypes[0].setHasMissing(value)
+        if not s is self.subtypes[0]:
+            self = self.copy()
+            self.subtypes = (s,)
+        return self
+
+
+    def toMissingval(self):
+        s = self.subtypes[0].toMissingval()
+        if self.dims[0].dependent:
+            res = numpy.array([],dtype=self.subtypes[0].toNumpy())
+        else:
+            if self.dims[0].shape == UNDEFINED:
+                raise RuntimeError, "Cannot determine shape for missing value. Please cast dim " + str(self.dims[0])
+            res = numpy.array([s] * self.dims[0].shape,dtype=self.toNumpy())
+        return res
+
+    def hasMissing(self):
+        return self.subtypes[0].hasMissing()
+
+
     def toDefval(self):
         """Returns default value."""
         subtype = self.subtypes[0]
-        if(not self.dims):
-            res = numpy.array([], dtype=subtype.toNumpy())
+        shape = [dim.shape for dim in self.dims]
+        for pos, sdim in enumerate(shape):
+            if(sdim < 0):
+                shape[pos] = 0
+        
+        res = numpy.empty(shape, dtype=subtype.toNumpy())
+        subdv = subtype.toDefval()
+        
+        #workaround numpy problem 
+        #(cannot set sequence for a range)
+        if(operator.isSequenceType(subdv)):
+            flatres = res.ravel()
+            for i in range(len(flatres)):
+                flatres[i] = subdv
         else:
-            shape = [dim.shape for dim in self.dims]
-            for pos, sdim in enumerate(shape):
-                if(sdim < 0):
-                    shape[pos] = 0
-            
-            res = numpy.empty(shape, dtype=subtype.toNumpy())
-            subdv = subtype.toDefval()
-           
-            #workaround numpy problem 
-            #(cannot set sequence for a range)
-            if(operator.isSequenceType(subdv)):
-                flatres = res.ravel()
-                for i in range(len(flatres)):
-                    flatres[i] = subdv
-            else:
-                res[:] = subdv
+            res[:] = subdv
         return res
     
 
@@ -759,6 +845,12 @@ class TypeSet(TypeArray):#{{{
         res += '{' + ",".join([str(dim) for dim in self.dims]) + '}'
         res += '<' + str(self.subtypes[0]) 
         return res
+    
+    def hasMissing(self):
+        return self.has_missing
+
+    def toMissingval(self):
+        return frozenset()
 
 addType(TypeSet)#}}}
 
@@ -791,10 +883,11 @@ class TypeString(TypeArray):#{{{
         """Returns dtype of a numpy container which
            can hold this type efficiently."""
 
-        if(self.dims[0].shape == UNDEFINED or self.has_missing or self.dims[0].shape > 32):
-            return numpy.dtype(object)
-        else:
-            return numpy.dtype(self._dtype + str(max(self.dims[0].shape,1)))
+        return numpy.dtype(object)
+        #f(self.dims[0].shape == UNDEFINED or self.has_missing or self.dims[0].shape > 32):
+        #   return numpy.dtype(object)
+        #lse:
+        #   return numpy.dtype(self._dtype + str(max(self.dims[0].shape,1)))
     
     def __eq__(self, other):
         return (self.__class__ is other.__class__ and 
@@ -805,7 +898,13 @@ class TypeString(TypeArray):#{{{
         return (hash(self.__class__) ^ 
                 hash(self.has_missing) ^ 
                 hash(self.dims[0].shape))
+
+    def toMissingval(self):
+        return Missing
     
+    def hasMissing(self):
+        return self.has_missing
+   
     def __repr__(self):
         res =  self.name
         if(self.has_missing):
@@ -1122,7 +1221,7 @@ class TypeStringScanner(GenericScanner):#{{{
         self.rv.append(t)
 
     def t_symbol(self,s):
-        r' \= | \$ | \? | \. | \{ | \} | \< | \[ | \] | \( | \) | \, | \* | \~ | \! | \& | \# | \@ | \: | \^ | \|'
+        r' \= | \$ | \? | \. | \{ | \} | \< | \[ | \] | \( | \) | \, | \* | \~ | \! | \& | \# | \@ | \: | \^ | \| | \' '
         t = Token(type=s)
         self.rv.append(t)#}}}
     
@@ -1219,7 +1318,7 @@ class TypeStringParser(GenericParser):#{{{
     
     def p_type_2b(self,args):
         """
-            type ::= type =
+            type ::= type '
         """
         return AST(type="strict", kids=args)
    
@@ -1248,10 +1347,12 @@ class TypeStringParser(GenericParser):#{{{
     def p_type_8(self,args):
         '''
             typeelem ::= typenest
-            typeelem ::= name : typenest
             typeelem ::= name = typenest
         '''
-        return AST(type="typeelem",kids=(args[2], args[0]))
+        if len(args) == 1:
+            return args[0]
+        else:
+            return AST(type="typeelem",kids=(args[2], args[0]))
     
     def p_type_9(self,args):
         ' type ::= ( typelist ) '
@@ -1606,7 +1707,22 @@ class TypeStringASTInterpreter(object):#{{{
         dimnode, has_missing, name, strict = node.kids
         if has_missing is None:
             has_missing = False
-       
+        
+        if(dimnode.type == "varlist"):
+            dependent = tuple(dimnode.kids)
+            shape = UNDEFINED
+        elif(dimnode.type == "integer"):
+            dependent = tuple()
+            shape = dimnode.attr
+        elif(dimnode.type == "~"):
+            dependent = (True,) * dimpos
+            shape = UNDEFINED
+        elif(dimnode.type == "inherit"):
+            dependent = None
+            shape = None
+        else:
+            raise RuntimeError, "Invalid AST!"
+      
         if(not name is None):
             assert name in self.dim_annot, "Unannotated named dim found!"
             if(not name in self.dims):
@@ -1623,19 +1739,11 @@ class TypeStringASTInterpreter(object):#{{{
             dim = self.dims[name]
             if(len(dim.dependent) > dimpos):
                 raise RuntimeError, "Dim: " + name + " has too many dependent dims: " + str(len(dim.dependent)) + " (max: " + str(dimpos) + ")"
+            if dim.shape != shape and not shape == UNDEFINED and not dim.shape == UNDEFINED:
+                raise RuntimeError, "Dim: " + name + " dimension unequal to known dim"
+
             return dim
 
-        if(dimnode.type == "varlist"):
-            dependent = tuple(dimnode.kids)
-            shape = UNDEFINED
-        elif(dimnode.type == "integer"):
-            dependent = tuple()
-            shape = dimnode.attr
-        elif(dimnode.type == "~"):
-            dependent = (True,) * dimpos
-            shape = UNDEFINED
-        else:
-            raise RuntimeError, "Invalid AST!"
 
         return dimensions.Dim(shape,dependent,has_missing, name=name) #}}}
 
@@ -1647,6 +1755,7 @@ def _createType(name, dimpos=0, refdims=[], env={}):
 
     tree = parser.parse(tokens)
     
+    #print tree
     rewriter1 = TypeStringASTRewriterPass1()
     tree, dim_annotation = rewriter1.process(tree)
     #print tree, dim_annotation
