@@ -221,9 +221,9 @@ class BinaryFuncElemOp(BinaryFuncOp):
                 name = binslices[0].name
             else:
                 name = "result"
+                if(not pos is None):
+                    outparam = outparam.withNumber(pos)
             outparam = Param(name, outparam)
-        if(not pos is None):
-            outparam = outparam.withNumber(pos)
         s = ops.BinFuncElemOp(cls.__name__, sig,outparam, **nkwargs)
         return s
 
@@ -485,6 +485,22 @@ class Abs(UnaryFuncElemOp):
 class Negative(UnaryFuncElemOp):
     _sigs = [unary_arithsig]
 
+
+class ReplaceMissingSig(FuncSignature):
+    def check(self, slice, def_value=NOVAL):#{{{
+        in_type = slice.type
+        nin_type = in_type.copy()
+        #FIXME: adapt dim in case of array
+        nin_type = nin_type.setHasMissing(False)
+
+        return Param(slice.name, nin_type)#}}}
+
+repmissing = ReplaceMissingSig("repmissing")
+
+class ReplaceMissing(UnaryFuncElemOp):
+    _sigs =[repmissing]
+
+
 class UnaryFixShapeSignature(FuncSignature):
     def __init__(self,name,check_dependent=True):
         self.check_dependent = check_dependent
@@ -630,6 +646,8 @@ class Argmin(UnaryFuncAggregateOp):
 class Median(UnaryFuncAggregateOp):
     _sigs = [number_tofloatsig]
 
+class Std(UnaryFuncAggregateOp):
+    _sigs = [number_tofloatsig]
 
 class CountSignature(FuncSignature):
     def check(self, slice, packdepth):#{{{
